@@ -23,7 +23,7 @@
  * Authors:
  *      Jerome Glisse
  */
-#include "r600_hw_context_priv.h"
+#include "r600_pipe.h"
 #include "r600d.h"
 #include "util/u_memory.h"
 #include <errno.h>
@@ -114,248 +114,6 @@ err:
 	return;
 }
 
-static void r600_init_block(struct r600_context *ctx,
-			    struct r600_block *block,
-			    const struct r600_reg *reg, int index, int nreg,
-			    unsigned opcode, unsigned offset_base)
-{
-	int i = index;
-	int j, n = nreg;
-
-	/* initialize block */
-	block->flags = 0;
-	block->status |= R600_BLOCK_STATUS_DIRTY; /* dirty all blocks at start */
-	block->start_offset = reg[i].offset;
-	block->pm4[block->pm4_ndwords++] = PKT3(opcode, n, 0);
-	block->pm4[block->pm4_ndwords++] = (block->start_offset - offset_base) >> 2;
-	block->reg = &block->pm4[block->pm4_ndwords];
-	block->pm4_ndwords += n;
-	block->nreg = n;
-	block->nreg_dirty = n;
-	LIST_INITHEAD(&block->list);
-	LIST_INITHEAD(&block->enable_list);
-
-	for (j = 0; j < n; j++) {
-		if (reg[i+j].flags & REG_FLAG_DIRTY_ALWAYS) {
-			block->flags |= REG_FLAG_DIRTY_ALWAYS;
-		}
-		if (reg[i+j].flags & REG_FLAG_ENABLE_ALWAYS) {
-			if (!(block->status & R600_BLOCK_STATUS_ENABLED)) {
-				block->status |= R600_BLOCK_STATUS_ENABLED;
-				LIST_ADDTAIL(&block->enable_list, &ctx->enable_list);
-				LIST_ADDTAIL(&block->list,&ctx->dirty);
-			}
-		}
-		if (reg[i+j].flags & REG_FLAG_FLUSH_CHANGE) {
-			block->flags |= REG_FLAG_FLUSH_CHANGE;
-		}
-
-		if (reg[i+j].flags & REG_FLAG_NEED_BO) {
-			block->nbo++;
-			assert(block->nbo < R600_BLOCK_MAX_BO);
-			block->pm4_bo_index[j] = block->nbo;
-			block->pm4[block->pm4_ndwords++] = PKT3(PKT3_NOP, 0, 0);
-			block->pm4[block->pm4_ndwords++] = 0x00000000;
-			block->reloc[block->nbo].bo_pm4_index = block->pm4_ndwords - 1;
-		}
-	}
-	/* check that we stay in limit */
-	assert(block->pm4_ndwords < R600_BLOCK_MAX_REG);
-}
-
-int r600_context_add_block(struct r600_context *ctx, const struct r600_reg *reg, unsigned nreg,
-			   unsigned opcode, unsigned offset_base)
-{
-	struct r600_block *block;
-	struct r600_range *range;
-	int offset;
-
-	for (unsigned i = 0, n = 0; i < nreg; i += n) {
-		/* ignore new block balise */
-		if (reg[i].offset == GROUP_FORCE_NEW_BLOCK) {
-			n = 1;
-			continue;
-		}
-
-		/* register that need relocation are in their own group */
-		/* find number of consecutive registers */
-		n = 0;
-		offset = reg[i].offset;
-		while (reg[i + n].offset == offset) {
-			n++;
-			offset += 4;
-			if ((n + i) >= nreg)
-				break;
-			if (n >= (R600_BLOCK_MAX_REG - 2))
-				break;
-		}
-
-		/* allocate new block */
-		block = calloc(1, sizeof(struct r600_block));
-		if (block == NULL) {
-			return -ENOMEM;
-		}
-		ctx->nblocks++;
-		for (int j = 0; j < n; j++) {
-			range = &ctx->range[CTX_RANGE_ID(reg[i + j].offset)];
-			/* create block table if it doesn't exist */
-			if (!range->blocks)
-				range->blocks = calloc(1 << HASH_SHIFT, sizeof(void *));
-			if (!range->blocks)
-				return -1;
-
-			range->blocks[CTX_BLOCK_ID(reg[i + j].offset)] = block;
-		}
-
-		r600_init_block(ctx, block, reg, i, n, opcode, offset_base);
-
-	}
-	return 0;
-}
-
-static const struct r600_reg r600_context_reg_list[] = {
-	{R_028D24_DB_HTILE_SURFACE, 0, 0},
-	{R_028614_SPI_VS_OUT_ID_0, 0, 0},
-	{R_028618_SPI_VS_OUT_ID_1, 0, 0},
-	{R_02861C_SPI_VS_OUT_ID_2, 0, 0},
-	{R_028620_SPI_VS_OUT_ID_3, 0, 0},
-	{R_028624_SPI_VS_OUT_ID_4, 0, 0},
-	{R_028628_SPI_VS_OUT_ID_5, 0, 0},
-	{R_02862C_SPI_VS_OUT_ID_6, 0, 0},
-	{R_028630_SPI_VS_OUT_ID_7, 0, 0},
-	{R_028634_SPI_VS_OUT_ID_8, 0, 0},
-	{R_028638_SPI_VS_OUT_ID_9, 0, 0},
-	{R_0286C4_SPI_VS_OUT_CONFIG, 0, 0},
-	{GROUP_FORCE_NEW_BLOCK, 0, 0},
-	{R_028858_SQ_PGM_START_VS, REG_FLAG_NEED_BO, 0},
-	{GROUP_FORCE_NEW_BLOCK, 0, 0},
-	{R_028868_SQ_PGM_RESOURCES_VS, 0, 0},
-	{GROUP_FORCE_NEW_BLOCK, 0, 0},
-	{R_0288A4_SQ_PGM_RESOURCES_FS, 0, 0},
-	{R_0288DC_SQ_PGM_CF_OFFSET_FS, 0, 0},
-	{R_028644_SPI_PS_INPUT_CNTL_0, 0, 0},
-	{R_028648_SPI_PS_INPUT_CNTL_1, 0, 0},
-	{R_02864C_SPI_PS_INPUT_CNTL_2, 0, 0},
-	{R_028650_SPI_PS_INPUT_CNTL_3, 0, 0},
-	{R_028654_SPI_PS_INPUT_CNTL_4, 0, 0},
-	{R_028658_SPI_PS_INPUT_CNTL_5, 0, 0},
-	{R_02865C_SPI_PS_INPUT_CNTL_6, 0, 0},
-	{R_028660_SPI_PS_INPUT_CNTL_7, 0, 0},
-	{R_028664_SPI_PS_INPUT_CNTL_8, 0, 0},
-	{R_028668_SPI_PS_INPUT_CNTL_9, 0, 0},
-	{R_02866C_SPI_PS_INPUT_CNTL_10, 0, 0},
-	{R_028670_SPI_PS_INPUT_CNTL_11, 0, 0},
-	{R_028674_SPI_PS_INPUT_CNTL_12, 0, 0},
-	{R_028678_SPI_PS_INPUT_CNTL_13, 0, 0},
-	{R_02867C_SPI_PS_INPUT_CNTL_14, 0, 0},
-	{R_028680_SPI_PS_INPUT_CNTL_15, 0, 0},
-	{R_028684_SPI_PS_INPUT_CNTL_16, 0, 0},
-	{R_028688_SPI_PS_INPUT_CNTL_17, 0, 0},
-	{R_02868C_SPI_PS_INPUT_CNTL_18, 0, 0},
-	{R_028690_SPI_PS_INPUT_CNTL_19, 0, 0},
-	{R_028694_SPI_PS_INPUT_CNTL_20, 0, 0},
-	{R_028698_SPI_PS_INPUT_CNTL_21, 0, 0},
-	{R_02869C_SPI_PS_INPUT_CNTL_22, 0, 0},
-	{R_0286A0_SPI_PS_INPUT_CNTL_23, 0, 0},
-	{R_0286A4_SPI_PS_INPUT_CNTL_24, 0, 0},
-	{R_0286A8_SPI_PS_INPUT_CNTL_25, 0, 0},
-	{R_0286AC_SPI_PS_INPUT_CNTL_26, 0, 0},
-	{R_0286B0_SPI_PS_INPUT_CNTL_27, 0, 0},
-	{R_0286B4_SPI_PS_INPUT_CNTL_28, 0, 0},
-	{R_0286B8_SPI_PS_INPUT_CNTL_29, 0, 0},
-	{R_0286BC_SPI_PS_INPUT_CNTL_30, 0, 0},
-	{R_0286C0_SPI_PS_INPUT_CNTL_31, 0, 0},
-	{R_0286CC_SPI_PS_IN_CONTROL_0, 0, 0},
-	{R_0286D0_SPI_PS_IN_CONTROL_1, 0, 0},
-	{R_0286D8_SPI_INPUT_Z, 0, 0},
-	{GROUP_FORCE_NEW_BLOCK, 0, 0},
-	{R_028840_SQ_PGM_START_PS, REG_FLAG_NEED_BO, 0},
-	{GROUP_FORCE_NEW_BLOCK, 0, 0},
-	{R_028850_SQ_PGM_RESOURCES_PS, 0, 0},
-	{R_028854_SQ_PGM_EXPORTS_PS, 0, 0},
-};
-
-/* initialize */
-void r600_context_fini(struct r600_context *ctx)
-{
-	struct r600_block *block;
-	struct r600_range *range;
-
-	if (ctx->range) {
-		for (int i = 0; i < NUM_RANGES; i++) {
-			if (!ctx->range[i].blocks)
-				continue;
-			for (int j = 0; j < (1 << HASH_SHIFT); j++) {
-				block = ctx->range[i].blocks[j];
-				if (block) {
-					for (int k = 0, offset = block->start_offset; k < block->nreg; k++, offset += 4) {
-						range = &ctx->range[CTX_RANGE_ID(offset)];
-						range->blocks[CTX_BLOCK_ID(offset)] = NULL;
-					}
-					for (int k = 1; k <= block->nbo; k++) {
-						pipe_resource_reference((struct pipe_resource**)&block->reloc[k].bo, NULL);
-					}
-					free(block);
-				}
-			}
-			free(ctx->range[i].blocks);
-		}
-	}
-	free(ctx->blocks);
-}
-
-int r600_setup_block_table(struct r600_context *ctx)
-{
-	/* setup block table */
-	int c = 0;
-	ctx->blocks = calloc(ctx->nblocks, sizeof(void*));
-	if (!ctx->blocks)
-		return -ENOMEM;
-	for (int i = 0; i < NUM_RANGES; i++) {
-		if (!ctx->range[i].blocks)
-			continue;
-		for (int j = 0, add; j < (1 << HASH_SHIFT); j++) {
-			if (!ctx->range[i].blocks[j])
-				continue;
-
-			add = 1;
-			for (int k = 0; k < c; k++) {
-				if (ctx->blocks[k] == ctx->range[i].blocks[j]) {
-					add = 0;
-					break;
-				}
-			}
-			if (add) {
-				assert(c < ctx->nblocks);
-				ctx->blocks[c++] = ctx->range[i].blocks[j];
-				j += (ctx->range[i].blocks[j]->nreg) - 1;
-			}
-		}
-	}
-	return 0;
-}
-
-int r600_context_init(struct r600_context *ctx)
-{
-	int r;
-
-	/* add blocks */
-	r = r600_context_add_block(ctx, r600_context_reg_list,
-				   Elements(r600_context_reg_list), PKT3_SET_CONTEXT_REG, R600_CONTEXT_REG_OFFSET);
-	if (r)
-		goto out_err;
-
-	r = r600_setup_block_table(ctx);
-	if (r)
-		goto out_err;
-
-	ctx->max_db = 4;
-	return 0;
-out_err:
-	r600_context_fini(ctx);
-	return r;
-}
-
 void r600_need_cs_space(struct r600_context *ctx, unsigned num_dw,
 			boolean count_draw_in)
 {
@@ -379,30 +137,26 @@ void r600_need_cs_space(struct r600_context *ctx, unsigned num_dw,
 		for (i = 0; i < R600_NUM_ATOMS; i++) {
 			if (ctx->atoms[i] && ctx->atoms[i]->dirty) {
 				num_dw += ctx->atoms[i]->num_dw;
-#if R600_TRACE_CS
 				if (ctx->screen->trace_bo) {
 					num_dw += R600_TRACE_CS_DWORDS;
 				}
-#endif
 			}
 		}
 
-		num_dw += ctx->pm4_dirty_cdwords;
-
 		/* The upper-bound of how much space a draw command would take. */
 		num_dw += R600_MAX_FLUSH_CS_DWORDS + R600_MAX_DRAW_CS_DWORDS;
-#if R600_TRACE_CS
 		if (ctx->screen->trace_bo) {
 			num_dw += R600_TRACE_CS_DWORDS;
 		}
-#endif
 	}
 
 	/* Count in queries_suspend. */
 	num_dw += ctx->num_cs_dw_nontimer_queries_suspend;
 
 	/* Count in streamout_end at the end of CS. */
-	num_dw += ctx->num_cs_dw_streamout_end;
+	if (ctx->streamout.begin_emitted) {
+		num_dw += ctx->streamout.num_dw_for_end;
+	}
 
 	/* Count in render_condition(NULL) at the end of CS. */
 	if (ctx->predicate_drawing) {
@@ -426,197 +180,11 @@ void r600_need_cs_space(struct r600_context *ctx, unsigned num_dw,
 	}
 }
 
-void r600_context_dirty_block(struct r600_context *ctx,
-			      struct r600_block *block,
-			      int dirty, int index)
-{
-	if ((index + 1) > block->nreg_dirty)
-		block->nreg_dirty = index + 1;
-
-	if ((dirty != (block->status & R600_BLOCK_STATUS_DIRTY)) || !(block->status & R600_BLOCK_STATUS_ENABLED)) {
-		block->status |= R600_BLOCK_STATUS_DIRTY;
-		ctx->pm4_dirty_cdwords += block->pm4_ndwords;
-		if (!(block->status & R600_BLOCK_STATUS_ENABLED)) {
-			block->status |= R600_BLOCK_STATUS_ENABLED;
-			LIST_ADDTAIL(&block->enable_list, &ctx->enable_list);
-		}
-		LIST_ADDTAIL(&block->list,&ctx->dirty);
-
-		if (block->flags & REG_FLAG_FLUSH_CHANGE) {
-			ctx->flags |= R600_CONTEXT_WAIT_3D_IDLE;
-		}
-	}
-}
-
-/**
- * If reg needs a reloc, this function will add it to its block's reloc list.
- * @return true if reg needs a reloc, false otherwise
- */
-static bool r600_reg_set_block_reloc(struct r600_pipe_reg *reg)
-{
-	unsigned reloc_id;
-
-	if (!reg->block->pm4_bo_index[reg->id]) {
-		return false;
-	}
-	/* find relocation */
-	reloc_id = reg->block->pm4_bo_index[reg->id];
-	pipe_resource_reference(
-		(struct pipe_resource**)&reg->block->reloc[reloc_id].bo,
-		&reg->bo->b.b);
-	reg->block->reloc[reloc_id].bo_usage = reg->bo_usage;
-	return true;
-}
-
-/**
- * This function will emit all the registers in state directly to the command
- * stream allowing you to bypass the r600_context dirty list.
- *
- * This is used for dispatching compute shaders to avoid mixing compute and
- * 3D states in the context's dirty list.
- *
- * @param pkt_flags Should be either 0 or RADEON_CP_PACKET3_COMPUTE_MODE.  This
- * value will be passed on to r600_context_block_emit_dirty an or'd against
- * the PKT3 headers.
- */
-void r600_context_pipe_state_emit(struct r600_context *ctx,
-                          struct r600_pipe_state *state,
-                          unsigned pkt_flags)
-{
-	unsigned i;
-
-	/* Mark all blocks as dirty: 
-	 * Since two registers can be in the same block, we need to make sure
-	 * we mark all the blocks dirty before we emit any of them.  If we were
-	 * to mark blocks dirty and emit them in the same loop, like this:
-	 *
-	 * foreach (reg in state->regs) {
-	 *     mark_dirty(reg->block)
-	 *     emit_block(reg->block)
-	 * }
-	 *
-	 * Then if we have two registers in this state that are in the same
-	 * block, we would end up emitting that block twice.
-	 */
-	for (i = 0; i < state->nregs; i++) {
-		struct r600_pipe_reg *reg = &state->regs[i];
-		/* Mark all the registers in the block as dirty */
-		reg->block->nreg_dirty = reg->block->nreg;
-		reg->block->status |= R600_BLOCK_STATUS_DIRTY;
-		/* Update the reloc for this register if necessary. */
-		r600_reg_set_block_reloc(reg);
-	}
-
-	/* Emit the registers writes */
-	for (i = 0; i < state->nregs; i++) {
-		struct r600_pipe_reg *reg = &state->regs[i];
-		if (reg->block->status & R600_BLOCK_STATUS_DIRTY) {
-			r600_context_block_emit_dirty(ctx, reg->block, pkt_flags);
-		}
-	}
-}
-
-void r600_context_pipe_state_set(struct r600_context *ctx, struct r600_pipe_state *state)
-{
-	struct r600_block *block;
-	int dirty;
-	for (int i = 0; i < state->nregs; i++) {
-		unsigned id;
-		struct r600_pipe_reg *reg = &state->regs[i];
-
-		block = reg->block;
-		id = reg->id;
-
-		dirty = block->status & R600_BLOCK_STATUS_DIRTY;
-
-		if (reg->value != block->reg[id]) {
-			block->reg[id] = reg->value;
-			dirty |= R600_BLOCK_STATUS_DIRTY;
-		}
-		if (block->flags & REG_FLAG_DIRTY_ALWAYS)
-			dirty |= R600_BLOCK_STATUS_DIRTY;
-		if (r600_reg_set_block_reloc(reg)) {
-			/* always force dirty for relocs for now */
-			dirty |= R600_BLOCK_STATUS_DIRTY;
-		}
-
-		if (dirty)
-			r600_context_dirty_block(ctx, block, dirty, id);
-	}
-}
-
-/**
- * @param pkt_flags should be set to RADEON_CP_PACKET3_COMPUTE_MODE if this
- * block will be used for compute shaders.
- */
-void r600_context_block_emit_dirty(struct r600_context *ctx, struct r600_block *block,
-	unsigned pkt_flags)
-{
-	struct radeon_winsys_cs *cs = ctx->rings.gfx.cs;
-	int optional = block->nbo == 0 && !(block->flags & REG_FLAG_DIRTY_ALWAYS);
-	int cp_dwords = block->pm4_ndwords, start_dword = 0;
-	int new_dwords = 0;
-	int nbo = block->nbo;
-
-	if (block->nreg_dirty == 0 && optional) {
-		goto out;
-	}
-
-	if (nbo) {
-		for (int j = 0; j < block->nreg; j++) {
-			if (block->pm4_bo_index[j]) {
-				/* find relocation */
-				struct r600_block_reloc *reloc = &block->reloc[block->pm4_bo_index[j]];
-				if (reloc->bo) {
-					block->pm4[reloc->bo_pm4_index] =
-							r600_context_bo_reloc(ctx, &ctx->rings.gfx, reloc->bo, reloc->bo_usage);
-				} else {
-					block->pm4[reloc->bo_pm4_index] = 0;
-				}
-				nbo--;
-				if (nbo == 0)
-					break;
-
-			}
-		}
-	}
-
-	optional &= (block->nreg_dirty != block->nreg);
-	if (optional) {
-		new_dwords = block->nreg_dirty;
-		start_dword = cs->cdw;
-		cp_dwords = new_dwords + 2;
-	}
-	memcpy(&cs->buf[cs->cdw], block->pm4, cp_dwords * 4);
-
-	/* We are applying the pkt_flags after copying the register block to
-	 * the the command stream, because it is possible this block will be
-	 * emitted with a different pkt_flags, and we don't want to store the
-	 * pkt_flags in the block.
-	 */
-	cs->buf[cs->cdw] |= pkt_flags;
-	cs->cdw += cp_dwords;
-
-	if (optional) {
-		uint32_t newword;
-
-		newword = cs->buf[start_dword];
-		newword &= PKT_COUNT_C;
-		newword |= PKT_COUNT_S(new_dwords);
-		cs->buf[start_dword] = newword;
-	}
-out:
-	block->status ^= R600_BLOCK_STATUS_DIRTY;
-	block->nreg_dirty = 0;
-	LIST_DELINIT(&block->list);
-}
-
 void r600_flush_emit(struct r600_context *rctx)
 {
 	struct radeon_winsys_cs *cs = rctx->rings.gfx.cs;
 	unsigned cp_coher_cntl = 0;
 	unsigned wait_until = 0;
-	unsigned emit_flush = 0;
 
 	if (!rctx->flags) {
 		return;
@@ -648,55 +216,77 @@ void r600_flush_emit(struct r600_context *rctx)
 		cs->buf[cs->cdw++] = EVENT_TYPE(EVENT_TYPE_FLUSH_AND_INV_CB_META) | EVENT_INDEX(0);
 	}
 
+	if (rctx->chip_class >= R700 &&
+	    (rctx->flags & R600_CONTEXT_FLUSH_AND_INV_DB_META)) {
+		cs->buf[cs->cdw++] = PKT3(PKT3_EVENT_WRITE, 0, 0);
+		cs->buf[cs->cdw++] = EVENT_TYPE(EVENT_TYPE_FLUSH_AND_INV_DB_META) | EVENT_INDEX(0);
+
+		/* Set FULL_CACHE_ENA for DB META flushes on r7xx and later.
+		 *
+		 * This hack predates use of FLUSH_AND_INV_DB_META, so it's
+		 * unclear whether it's still needed or even whether it has
+		 * any effect.
+		 */
+		cp_coher_cntl |= S_0085F0_FULL_CACHE_ENA(1);
+	}
+
 	if (rctx->flags & R600_CONTEXT_FLUSH_AND_INV) {
 		cs->buf[cs->cdw++] = PKT3(PKT3_EVENT_WRITE, 0, 0);
 		cs->buf[cs->cdw++] = EVENT_TYPE(EVENT_TYPE_CACHE_FLUSH_AND_INV_EVENT) | EVENT_INDEX(0);
-		if (rctx->chip_class >= EVERGREEN) {
-			cp_coher_cntl = S_0085F0_CB0_DEST_BASE_ENA(1) |
-					S_0085F0_CB1_DEST_BASE_ENA(1) |
-					S_0085F0_CB2_DEST_BASE_ENA(1) |
-					S_0085F0_CB3_DEST_BASE_ENA(1) |
-					S_0085F0_CB4_DEST_BASE_ENA(1) |
-					S_0085F0_CB5_DEST_BASE_ENA(1) |
-					S_0085F0_CB6_DEST_BASE_ENA(1) |
-					S_0085F0_CB7_DEST_BASE_ENA(1) |
-					S_0085F0_CB8_DEST_BASE_ENA(1) |
+	}
+
+	if (rctx->flags & R600_CONTEXT_INV_CONST_CACHE) {
+		cp_coher_cntl |= S_0085F0_SH_ACTION_ENA(1);
+	}
+	if (rctx->flags & R600_CONTEXT_INV_VERTEX_CACHE) {
+		cp_coher_cntl |= rctx->has_vertex_cache ? S_0085F0_VC_ACTION_ENA(1)
+							: S_0085F0_TC_ACTION_ENA(1);
+	}
+	if (rctx->flags & R600_CONTEXT_INV_TEX_CACHE) {
+		cp_coher_cntl |= S_0085F0_TC_ACTION_ENA(1);
+	}
+
+	/* Don't use the DB CP COHER logic on r6xx.
+	 * There are hw bugs.
+	 */
+	if (rctx->chip_class >= R700 &&
+	    (rctx->flags & R600_CONTEXT_FLUSH_AND_INV_DB)) {
+		cp_coher_cntl |= S_0085F0_DB_ACTION_ENA(1) |
+				S_0085F0_DB_DEST_BASE_ENA(1) |
+				S_0085F0_SMX_ACTION_ENA(1);
+	}
+
+	/* Don't use the CB CP COHER logic on r6xx.
+	 * There are hw bugs.
+	 */
+	if (rctx->chip_class >= R700 &&
+	    (rctx->flags & R600_CONTEXT_FLUSH_AND_INV_CB)) {
+		cp_coher_cntl |= S_0085F0_CB_ACTION_ENA(1) |
+				S_0085F0_CB0_DEST_BASE_ENA(1) |
+				S_0085F0_CB1_DEST_BASE_ENA(1) |
+				S_0085F0_CB2_DEST_BASE_ENA(1) |
+				S_0085F0_CB3_DEST_BASE_ENA(1) |
+				S_0085F0_CB4_DEST_BASE_ENA(1) |
+				S_0085F0_CB5_DEST_BASE_ENA(1) |
+				S_0085F0_CB6_DEST_BASE_ENA(1) |
+				S_0085F0_CB7_DEST_BASE_ENA(1) |
+				S_0085F0_SMX_ACTION_ENA(1);
+		if (rctx->chip_class >= EVERGREEN)
+			cp_coher_cntl |= S_0085F0_CB8_DEST_BASE_ENA(1) |
 					S_0085F0_CB9_DEST_BASE_ENA(1) |
 					S_0085F0_CB10_DEST_BASE_ENA(1) |
-					S_0085F0_CB11_DEST_BASE_ENA(1) |
-					S_0085F0_DB_DEST_BASE_ENA(1) |
-					S_0085F0_TC_ACTION_ENA(1) |
-					S_0085F0_CB_ACTION_ENA(1) |
-					S_0085F0_DB_ACTION_ENA(1) |
-					S_0085F0_SH_ACTION_ENA(1) |
-					S_0085F0_SMX_ACTION_ENA(1) |
-					S_0085F0_FULL_CACHE_ENA(1);
-		} else {
-			cp_coher_cntl = S_0085F0_SMX_ACTION_ENA(1) |
-					S_0085F0_SH_ACTION_ENA(1) |
-					S_0085F0_VC_ACTION_ENA(1) |
-					S_0085F0_TC_ACTION_ENA(1) |
-					S_0085F0_FULL_CACHE_ENA(1);
-		}
+					S_0085F0_CB11_DEST_BASE_ENA(1);
 	}
 
-	if (rctx->flags & R600_CONTEXT_INVAL_READ_CACHES) {
-		cp_coher_cntl |= S_0085F0_VC_ACTION_ENA(1) |
-				S_0085F0_TC_ACTION_ENA(1) |
-				S_0085F0_FULL_CACHE_ENA(1);
-		emit_flush = 1;
-	}
-
-	if (rctx->family >= CHIP_RV770 && rctx->flags & R600_CONTEXT_STREAMOUT_FLUSH) {
+	if (rctx->flags & R600_CONTEXT_STREAMOUT_FLUSH) {
 		cp_coher_cntl |= S_0085F0_SO0_DEST_BASE_ENA(1) |
 				S_0085F0_SO1_DEST_BASE_ENA(1) |
 				S_0085F0_SO2_DEST_BASE_ENA(1) |
 				S_0085F0_SO3_DEST_BASE_ENA(1) |
 				S_0085F0_SMX_ACTION_ENA(1);
-		emit_flush = 1;
 	}
 
-	if (emit_flush) {
+	if (cp_coher_cntl) {
 		cs->buf[cs->cdw++] = PKT3(PKT3_SURFACE_SYNC, 3, 0);
 		cs->buf[cs->cdw++] = cp_coher_cntl;   /* CP_COHER_CNTL */
 		cs->buf[cs->cdw++] = 0xffffffff;      /* CP_COHER_SIZE */
@@ -720,11 +310,8 @@ void r600_context_flush(struct r600_context *ctx, unsigned flags)
 {
 	struct radeon_winsys_cs *cs = ctx->rings.gfx.cs;
 
-	if (cs->cdw == ctx->start_cs_cmd.num_dw)
-		return;
-
 	ctx->nontimer_queries_suspended = false;
-	ctx->streamout_suspended = false;
+	ctx->streamout.suspended = false;
 
 	/* suspend queries */
 	if (ctx->num_cs_dw_nontimer_queries_suspend) {
@@ -732,16 +319,19 @@ void r600_context_flush(struct r600_context *ctx, unsigned flags)
 		ctx->nontimer_queries_suspended = true;
 	}
 
-	if (ctx->num_cs_dw_streamout_end) {
-		r600_context_streamout_end(ctx);
-		ctx->streamout_suspended = true;
+	if (ctx->streamout.begin_emitted) {
+		r600_emit_streamout_end(ctx);
+		ctx->streamout.suspended = true;
 	}
 
 	/* flush is needed to avoid lockups on some chips with user fences
 	 * this will also flush the framebuffer cache
 	 */
 	ctx->flags |= R600_CONTEXT_FLUSH_AND_INV |
+		      R600_CONTEXT_FLUSH_AND_INV_CB |
+		      R600_CONTEXT_FLUSH_AND_INV_DB |
 		      R600_CONTEXT_FLUSH_AND_INV_CB_META |
+		      R600_CONTEXT_FLUSH_AND_INV_DB_META |
 		      R600_CONTEXT_WAIT_3D_IDLE |
 		      R600_CONTEXT_WAIT_CP_DMA_IDLE;
 
@@ -758,45 +348,13 @@ void r600_context_flush(struct r600_context *ctx, unsigned flags)
 	}
 
 	/* Flush the CS. */
-#if R600_TRACE_CS
-	if (ctx->screen->trace_bo) {
-		struct r600_screen *rscreen = ctx->screen;
-		unsigned i;
-
-		for (i = 0; i < cs->cdw; i++) {
-			fprintf(stderr, "[%4d] [%5d] 0x%08x\n", rscreen->cs_count, i, cs->buf[i]);
-		}
-		rscreen->cs_count++;
-	}
-#endif
-	ctx->ws->cs_flush(ctx->rings.gfx.cs, flags);
-#if R600_TRACE_CS
-	if (ctx->screen->trace_bo) {
-		struct r600_screen *rscreen = ctx->screen;
-		unsigned i;
-
-		for (i = 0; i < 10; i++) {
-			usleep(5);
-			if (!ctx->ws->buffer_is_busy(rscreen->trace_bo->buf, RADEON_USAGE_READWRITE)) {
-				break;
-			}
-		}
-		if (i == 10) {
-			fprintf(stderr, "timeout on cs lockup likely happen at cs %d dw %d\n",
-				rscreen->trace_ptr[1], rscreen->trace_ptr[0]);
-		} else {
-			fprintf(stderr, "cs %d executed in %dms\n", rscreen->trace_ptr[1], i * 5);
-		}
-	}
-#endif
+	ctx->ws->cs_flush(ctx->rings.gfx.cs, flags, ctx->screen->cs_count++);
 }
 
 void r600_begin_new_cs(struct r600_context *ctx)
 {
-	struct r600_block *enable_block = NULL;
 	unsigned shader;
 
-	ctx->pm4_dirty_cdwords = 0;
 	ctx->flags = 0;
 	ctx->gtt = 0;
 	ctx->vram = 0;
@@ -813,14 +371,15 @@ void r600_begin_new_cs(struct r600_context *ctx)
 	ctx->db_misc_state.atom.dirty = true;
 	ctx->db_state.atom.dirty = true;
 	ctx->framebuffer.atom.dirty = true;
+	ctx->pixel_shader.atom.dirty = true;
 	ctx->poly_offset_state.atom.dirty = true;
 	ctx->vgt_state.atom.dirty = true;
-	ctx->vgt2_state.atom.dirty = true;
 	ctx->sample_mask.atom.dirty = true;
 	ctx->scissor.atom.dirty = true;
 	ctx->config_state.atom.dirty = true;
 	ctx->stencil_ref.atom.dirty = true;
 	ctx->vertex_fetch_shader.atom.dirty = true;
+	ctx->vertex_shader.atom.dirty = true;
 	ctx->viewport.atom.dirty = true;
 
 	if (ctx->blend_state.cso)
@@ -851,9 +410,9 @@ void r600_begin_new_cs(struct r600_context *ctx)
 		r600_sampler_states_dirty(ctx, &samplers->states);
 	}
 
-	if (ctx->streamout_suspended) {
-		ctx->streamout_start = TRUE;
-		ctx->streamout_append_bitmask = ~0;
+	if (ctx->streamout.suspended) {
+		ctx->streamout.append_bitmask = ctx->streamout.enabled_mask;
+		r600_streamout_buffers_dirty(ctx);
 	}
 
 	/* resume queries */
@@ -861,21 +420,11 @@ void r600_begin_new_cs(struct r600_context *ctx)
 		r600_resume_nontimer_queries(ctx);
 	}
 
-	/* set all valid group as dirty so they get reemited on
-	 * next draw command
-	 */
-	LIST_FOR_EACH_ENTRY(enable_block, &ctx->enable_list, enable_list) {
-		if(!(enable_block->status & R600_BLOCK_STATUS_DIRTY)) {
-			LIST_ADDTAIL(&enable_block->list,&ctx->dirty);
-			enable_block->status |= R600_BLOCK_STATUS_DIRTY;
-		}
-		ctx->pm4_dirty_cdwords += enable_block->pm4_ndwords;
-		enable_block->nreg_dirty = enable_block->nreg;
-	}
-
 	/* Re-emit the draw state. */
 	ctx->last_primitive_type = -1;
 	ctx->last_start_instance = -1;
+
+	ctx->initial_gfx_cs_size = ctx->rings.gfx.cs->cdw;
 }
 
 void r600_context_emit_fence(struct r600_context *ctx, struct r600_resource *fence_bo, unsigned offset, unsigned value)
@@ -937,48 +486,23 @@ static void r600_set_streamout_enable(struct r600_context *ctx, unsigned buffer_
 	}
 }
 
-void r600_context_streamout_begin(struct r600_context *ctx)
+void r600_emit_streamout_begin(struct r600_context *ctx, struct r600_atom *atom)
 {
 	struct radeon_winsys_cs *cs = ctx->rings.gfx.cs;
-	struct r600_so_target **t = ctx->so_targets;
+	struct r600_so_target **t = ctx->streamout.targets;
 	unsigned *stride_in_dw = ctx->vs_shader->so.stride;
-	unsigned buffer_en, i, update_flags = 0;
+	unsigned i, update_flags = 0;
 	uint64_t va;
-	unsigned num_cs_dw_streamout_end;
-
-	buffer_en = (ctx->num_so_targets >= 1 && t[0] ? 1 : 0) |
-		    (ctx->num_so_targets >= 2 && t[1] ? 2 : 0) |
-		    (ctx->num_so_targets >= 3 && t[2] ? 4 : 0) |
-		    (ctx->num_so_targets >= 4 && t[3] ? 8 : 0);
-
-	num_cs_dw_streamout_end =
-		12 + /* flush_vgt_streamout */
-		util_bitcount(buffer_en) * 8 + /* STRMOUT_BUFFER_UPDATE */
-		3 /* set_streamout_enable(0) */;
-
-	r600_need_cs_space(ctx,
-			   12 + /* flush_vgt_streamout */
-			   6 + /* set_streamout_enable */
-			   util_bitcount(buffer_en) * 7 + /* SET_CONTEXT_REG */
-			   (ctx->family >= CHIP_RS780 &&
-			    ctx->family <= CHIP_RV740 ? util_bitcount(buffer_en) * 5 : 0) + /* STRMOUT_BASE_UPDATE */
-			   util_bitcount(buffer_en & ctx->streamout_append_bitmask) * 8 + /* STRMOUT_BUFFER_UPDATE */
-			   util_bitcount(buffer_en & ~ctx->streamout_append_bitmask) * 6 + /* STRMOUT_BUFFER_UPDATE */
-			   (ctx->family > CHIP_R600 && ctx->family < CHIP_RS780 ? 2 : 0) + /* SURFACE_BASE_UPDATE */
-			   num_cs_dw_streamout_end, TRUE);
-
-	/* This must be set after r600_need_cs_space. */
-	ctx->num_cs_dw_streamout_end = num_cs_dw_streamout_end;
 
 	if (ctx->chip_class >= EVERGREEN) {
 		evergreen_flush_vgt_streamout(ctx);
-		evergreen_set_streamout_enable(ctx, buffer_en);
+		evergreen_set_streamout_enable(ctx, ctx->streamout.enabled_mask);
 	} else {
 		r600_flush_vgt_streamout(ctx);
-		r600_set_streamout_enable(ctx, buffer_en);
+		r600_set_streamout_enable(ctx, ctx->streamout.enabled_mask);
 	}
 
-	for (i = 0; i < ctx->num_so_targets; i++) {
+	for (i = 0; i < ctx->streamout.num_targets; i++) {
 		if (t[i]) {
 			t[i]->stride_in_dw = stride_in_dw[i];
 			t[i]->so_index = i;
@@ -1011,7 +535,7 @@ void r600_context_streamout_begin(struct r600_context *ctx)
 							      RADEON_USAGE_WRITE);
 			}
 
-			if (ctx->streamout_append_bitmask & (1 << i)) {
+			if (ctx->streamout.append_bitmask & (1 << i)) {
 				va = r600_resource_va(&ctx->screen->screen,
 						      (void*)t[i]->buf_filled_size) + t[i]->buf_filled_size_offset;
 				/* Append. */
@@ -1040,16 +564,17 @@ void r600_context_streamout_begin(struct r600_context *ctx)
 		}
 	}
 
-	if (ctx->family > CHIP_R600 && ctx->family < CHIP_RS780) {
+	if (ctx->family > CHIP_R600 && ctx->family < CHIP_RV770) {
 		cs->buf[cs->cdw++] = PKT3(PKT3_SURFACE_BASE_UPDATE, 0, 0);
 		cs->buf[cs->cdw++] = update_flags;
 	}
+	ctx->streamout.begin_emitted = true;
 }
 
-void r600_context_streamout_end(struct r600_context *ctx)
+void r600_emit_streamout_end(struct r600_context *ctx)
 {
 	struct radeon_winsys_cs *cs = ctx->rings.gfx.cs;
-	struct r600_so_target **t = ctx->so_targets;
+	struct r600_so_target **t = ctx->streamout.targets;
 	unsigned i;
 	uint64_t va;
 
@@ -1059,7 +584,7 @@ void r600_context_streamout_end(struct r600_context *ctx)
 		r600_flush_vgt_streamout(ctx);
 	}
 
-	for (i = 0; i < ctx->num_so_targets; i++) {
+	for (i = 0; i < ctx->streamout.num_targets; i++) {
 		if (t[i]) {
 			va = r600_resource_va(&ctx->screen->screen,
 					      (void*)t[i]->buf_filled_size) + t[i]->buf_filled_size_offset;
@@ -1076,7 +601,6 @@ void r600_context_streamout_end(struct r600_context *ctx)
 			cs->buf[cs->cdw++] =
 				r600_context_bo_reloc(ctx,  &ctx->rings.gfx, t[i]->buf_filled_size,
 						      RADEON_USAGE_WRITE);
-
 		}
 	}
 
@@ -1090,7 +614,7 @@ void r600_context_streamout_end(struct r600_context *ctx)
 		r600_set_streamout_enable(ctx, 0);
 	}
 	ctx->flags |= R600_CONTEXT_WAIT_3D_IDLE | R600_CONTEXT_FLUSH_AND_INV;
-	ctx->num_cs_dw_streamout_end = 0;
+	ctx->streamout.begin_emitted = false;
 }
 
 /* The max number of bytes to copy per packet. */
@@ -1104,23 +628,15 @@ void r600_cp_dma_copy_buffer(struct r600_context *rctx,
 	struct radeon_winsys_cs *cs = rctx->rings.gfx.cs;
 
 	assert(size);
-	assert(rctx->chip_class != R600);
-
-	/* CP DMA doesn't work on R600 (flushing seems to be unreliable). */
-	if (rctx->chip_class == R600) {
-		return;
-	}
+	assert(rctx->screen->has_cp_dma);
 
 	dst_offset += r600_resource_va(&rctx->screen->screen, dst);
 	src_offset += r600_resource_va(&rctx->screen->screen, src);
 
-	/* We flush the caches, because we might read from or write
-	 * to resources which are bound right now. */
-	rctx->flags |= R600_CONTEXT_INVAL_READ_CACHES |
-		       R600_CONTEXT_FLUSH_AND_INV |
-		       R600_CONTEXT_FLUSH_AND_INV_CB_META |
-		       R600_CONTEXT_STREAMOUT_FLUSH |
-		       R600_CONTEXT_WAIT_3D_IDLE;
+	/* Flush the caches where the resources are bound. */
+	r600_flag_resource_cache_flush(rctx, src);
+	r600_flag_resource_cache_flush(rctx, dst);
+        rctx->flags |= R600_CONTEXT_WAIT_3D_IDLE;
 
 	/* There are differences between R700 and EG in CP DMA,
 	 * but we only use the common bits here. */
@@ -1162,8 +678,12 @@ void r600_cp_dma_copy_buffer(struct r600_context *rctx,
 		dst_offset += byte_count;
 	}
 
-	/* Invalidate the read caches. */
-	rctx->flags |= R600_CONTEXT_INVAL_READ_CACHES;
+	/* Flush the cache of the dst resource again in case the 3D engine
+	 * has been prefetching it. */
+	r600_flag_resource_cache_flush(rctx, dst);
+
+	util_range_add(&r600_resource(dst)->valid_buffer_range, dst_offset,
+		       dst_offset + size);
 }
 
 void r600_need_dma_space(struct r600_context *ctx, unsigned num_dw)
@@ -1210,4 +730,111 @@ void r600_dma_copy(struct r600_context *rctx,
 		src_offset += csize << shift;
 		size -= csize;
 	}
+
+	util_range_add(&rdst->valid_buffer_range, dst_offset,
+		       dst_offset + size);
+}
+
+/* Flag the cache of the resource for it to be flushed later if the resource
+ * is bound. Otherwise do nothing. Used for synchronization between engines.
+ */
+void r600_flag_resource_cache_flush(struct r600_context *rctx,
+				    struct pipe_resource *res)
+{
+	/* Check vertex buffers. */
+	uint32_t mask = rctx->vertex_buffer_state.enabled_mask;
+	while (mask) {
+		uint32_t i = u_bit_scan(&mask);
+		if (rctx->vertex_buffer_state.vb[i].buffer == res) {
+			rctx->flags |= R600_CONTEXT_INV_VERTEX_CACHE;
+		}
+	}
+
+	/* Check vertex buffers for compute. */
+	mask = rctx->cs_vertex_buffer_state.enabled_mask;
+	while (mask) {
+		uint32_t i = u_bit_scan(&mask);
+		if (rctx->cs_vertex_buffer_state.vb[i].buffer == res) {
+			rctx->flags |= R600_CONTEXT_INV_VERTEX_CACHE;
+		}
+	}
+
+	/* Check constant buffers. */
+	unsigned shader;
+	for (shader = 0; shader < PIPE_SHADER_TYPES; shader++) {
+		struct r600_constbuf_state *state = &rctx->constbuf_state[shader];
+		uint32_t mask = state->enabled_mask;
+
+		while (mask) {
+			unsigned i = u_bit_scan(&mask);
+			if (state->cb[i].buffer == res) {
+				rctx->flags |= R600_CONTEXT_INV_CONST_CACHE;
+
+				shader = PIPE_SHADER_TYPES; /* break the outer loop */
+				break;
+			}
+		}
+	}
+
+	/* Check textures. */
+	for (shader = 0; shader < PIPE_SHADER_TYPES; shader++) {
+		struct r600_samplerview_state *state = &rctx->samplers[shader].views;
+		uint32_t mask = state->enabled_mask;
+
+		while (mask) {
+			uint32_t i = u_bit_scan(&mask);
+			if (&state->views[i]->tex_resource->b.b == res) {
+				rctx->flags |= R600_CONTEXT_INV_TEX_CACHE;
+
+				shader = PIPE_SHADER_TYPES; /* break the outer loop */
+				break;
+			}
+		}
+	}
+
+	/* Check streamout buffers. */
+	int i;
+	for (i = 0; i < rctx->streamout.num_targets; i++) {
+		if (rctx->streamout.targets[i]->b.buffer == res) {
+			rctx->flags |= R600_CONTEXT_STREAMOUT_FLUSH |
+				       R600_CONTEXT_FLUSH_AND_INV |
+				       R600_CONTEXT_WAIT_3D_IDLE;
+			break;
+		}
+	}
+
+	/* Check colorbuffers. */
+	for (i = 0; i < rctx->framebuffer.state.nr_cbufs; i++) {
+		if (rctx->framebuffer.state.cbufs[i] &&
+		    rctx->framebuffer.state.cbufs[i]->texture == res) {
+			struct r600_texture *tex =
+				(struct r600_texture*)rctx->framebuffer.state.cbufs[i]->texture;
+
+			rctx->flags |= R600_CONTEXT_FLUSH_AND_INV_CB |
+				       R600_CONTEXT_FLUSH_AND_INV |
+				       R600_CONTEXT_WAIT_3D_IDLE;
+
+			if (tex->cmask_size || tex->fmask_size) {
+				rctx->flags |= R600_CONTEXT_FLUSH_AND_INV_CB_META;
+			}
+			break;
+		}
+	}
+
+	/* Check a depth buffer. */
+	if (rctx->framebuffer.state.zsbuf) {
+		if (rctx->framebuffer.state.zsbuf->texture == res) {
+			rctx->flags |= R600_CONTEXT_FLUSH_AND_INV_DB |
+				       R600_CONTEXT_FLUSH_AND_INV |
+				       R600_CONTEXT_WAIT_3D_IDLE;
+		}
+
+		struct r600_texture *tex =
+			(struct r600_texture*)rctx->framebuffer.state.zsbuf->texture;
+		if (tex && tex->htile && &tex->htile->b.b == res) {
+			rctx->flags |= R600_CONTEXT_FLUSH_AND_INV_DB_META |
+				       R600_CONTEXT_FLUSH_AND_INV |
+				       R600_CONTEXT_WAIT_3D_IDLE;
+		}
+	}
 }
