@@ -523,40 +523,40 @@ const struct name_offset linux_gl_abi[] = {
    { "glTexImage3D", 371 },
    { "glTexSubImage3D", 372 },
    { "glCopyTexSubImage3D", 373 },
-   { "glActiveTextureARB", 374 },
-   { "glClientActiveTextureARB", 375 },
-   { "glMultiTexCoord1dARB", 376 },
-   { "glMultiTexCoord1dvARB", 377 },
+   { "glActiveTexture", 374 },
+   { "glClientActiveTexture", 375 },
+   { "glMultiTexCoord1d", 376 },
+   { "glMultiTexCoord1dv", 377 },
    { "glMultiTexCoord1fARB", 378 },
    { "glMultiTexCoord1fvARB", 379 },
-   { "glMultiTexCoord1iARB", 380 },
-   { "glMultiTexCoord1ivARB", 381 },
-   { "glMultiTexCoord1sARB", 382 },
-   { "glMultiTexCoord1svARB", 383 },
-   { "glMultiTexCoord2dARB", 384 },
-   { "glMultiTexCoord2dvARB", 385 },
+   { "glMultiTexCoord1i", 380 },
+   { "glMultiTexCoord1iv", 381 },
+   { "glMultiTexCoord1s", 382 },
+   { "glMultiTexCoord1sv", 383 },
+   { "glMultiTexCoord2d", 384 },
+   { "glMultiTexCoord2dv", 385 },
    { "glMultiTexCoord2fARB", 386 },
    { "glMultiTexCoord2fvARB", 387 },
-   { "glMultiTexCoord2iARB", 388 },
-   { "glMultiTexCoord2ivARB", 389 },
-   { "glMultiTexCoord2sARB", 390 },
-   { "glMultiTexCoord2svARB", 391 },
-   { "glMultiTexCoord3dARB", 392 },
-   { "glMultiTexCoord3dvARB", 393 },
+   { "glMultiTexCoord2i", 388 },
+   { "glMultiTexCoord2iv", 389 },
+   { "glMultiTexCoord2s", 390 },
+   { "glMultiTexCoord2sv", 391 },
+   { "glMultiTexCoord3d", 392 },
+   { "glMultiTexCoord3dv", 393 },
    { "glMultiTexCoord3fARB", 394 },
    { "glMultiTexCoord3fvARB", 395 },
-   { "glMultiTexCoord3iARB", 396 },
-   { "glMultiTexCoord3ivARB", 397 },
-   { "glMultiTexCoord3sARB", 398 },
-   { "glMultiTexCoord3svARB", 399 },
-   { "glMultiTexCoord4dARB", 400 },
-   { "glMultiTexCoord4dvARB", 401 },
+   { "glMultiTexCoord3i", 396 },
+   { "glMultiTexCoord3iv", 397 },
+   { "glMultiTexCoord3s", 398 },
+   { "glMultiTexCoord3sv", 399 },
+   { "glMultiTexCoord4d", 400 },
+   { "glMultiTexCoord4dv", 401 },
    { "glMultiTexCoord4fARB", 402 },
    { "glMultiTexCoord4fvARB", 403 },
-   { "glMultiTexCoord4iARB", 404 },
-   { "glMultiTexCoord4ivARB", 405 },
-   { "glMultiTexCoord4sARB", 406 },
-   { "glMultiTexCoord4svARB", 407 },
+   { "glMultiTexCoord4i", 404 },
+   { "glMultiTexCoord4iv", 405 },
+   { "glMultiTexCoord4s", 406 },
+   { "glMultiTexCoord4sv", 407 },
    { NULL, 0 }
 };
 
@@ -937,40 +937,40 @@ const struct name_offset known_dispatch[] = {
    { "glTexImage3D", _O(TexImage3D) },
    { "glTexSubImage3D", _O(TexSubImage3D) },
    { "glCopyTexSubImage3D", _O(CopyTexSubImage3D) },
-   { "glActiveTextureARB", _O(ActiveTextureARB) },
-   { "glClientActiveTextureARB", _O(ClientActiveTextureARB) },
-   { "glMultiTexCoord1dARB", _O(MultiTexCoord1dARB) },
-   { "glMultiTexCoord1dvARB", _O(MultiTexCoord1dvARB) },
+   { "glActiveTexture", _O(ActiveTexture) },
+   { "glClientActiveTexture", _O(ClientActiveTexture) },
+   { "glMultiTexCoord1d", _O(MultiTexCoord1d) },
+   { "glMultiTexCoord1dv", _O(MultiTexCoord1dv) },
    { "glMultiTexCoord1fARB", _O(MultiTexCoord1fARB) },
    { "glMultiTexCoord1fvARB", _O(MultiTexCoord1fvARB) },
-   { "glMultiTexCoord1iARB", _O(MultiTexCoord1iARB) },
-   { "glMultiTexCoord1ivARB", _O(MultiTexCoord1ivARB) },
-   { "glMultiTexCoord1sARB", _O(MultiTexCoord1sARB) },
-   { "glMultiTexCoord1svARB", _O(MultiTexCoord1svARB) },
-   { "glMultiTexCoord2dARB", _O(MultiTexCoord2dARB) },
-   { "glMultiTexCoord2dvARB", _O(MultiTexCoord2dvARB) },
+   { "glMultiTexCoord1i", _O(MultiTexCoord1i) },
+   { "glMultiTexCoord1iv", _O(MultiTexCoord1iv) },
+   { "glMultiTexCoord1s", _O(MultiTexCoord1s) },
+   { "glMultiTexCoord1sv", _O(MultiTexCoord1sv) },
+   { "glMultiTexCoord2d", _O(MultiTexCoord2d) },
+   { "glMultiTexCoord2dv", _O(MultiTexCoord2dv) },
    { "glMultiTexCoord2fARB", _O(MultiTexCoord2fARB) },
    { "glMultiTexCoord2fvARB", _O(MultiTexCoord2fvARB) },
-   { "glMultiTexCoord2iARB", _O(MultiTexCoord2iARB) },
-   { "glMultiTexCoord2ivARB", _O(MultiTexCoord2ivARB) },
-   { "glMultiTexCoord2sARB", _O(MultiTexCoord2sARB) },
-   { "glMultiTexCoord2svARB", _O(MultiTexCoord2svARB) },
-   { "glMultiTexCoord3dARB", _O(MultiTexCoord3dARB) },
-   { "glMultiTexCoord3dvARB", _O(MultiTexCoord3dvARB) },
+   { "glMultiTexCoord2i", _O(MultiTexCoord2i) },
+   { "glMultiTexCoord2iv", _O(MultiTexCoord2iv) },
+   { "glMultiTexCoord2s", _O(MultiTexCoord2s) },
+   { "glMultiTexCoord2sv", _O(MultiTexCoord2sv) },
+   { "glMultiTexCoord3d", _O(MultiTexCoord3d) },
+   { "glMultiTexCoord3dv", _O(MultiTexCoord3dv) },
    { "glMultiTexCoord3fARB", _O(MultiTexCoord3fARB) },
    { "glMultiTexCoord3fvARB", _O(MultiTexCoord3fvARB) },
-   { "glMultiTexCoord3iARB", _O(MultiTexCoord3iARB) },
-   { "glMultiTexCoord3ivARB", _O(MultiTexCoord3ivARB) },
-   { "glMultiTexCoord3sARB", _O(MultiTexCoord3sARB) },
-   { "glMultiTexCoord3svARB", _O(MultiTexCoord3svARB) },
-   { "glMultiTexCoord4dARB", _O(MultiTexCoord4dARB) },
-   { "glMultiTexCoord4dvARB", _O(MultiTexCoord4dvARB) },
+   { "glMultiTexCoord3i", _O(MultiTexCoord3i) },
+   { "glMultiTexCoord3iv", _O(MultiTexCoord3iv) },
+   { "glMultiTexCoord3s", _O(MultiTexCoord3s) },
+   { "glMultiTexCoord3sv", _O(MultiTexCoord3sv) },
+   { "glMultiTexCoord4d", _O(MultiTexCoord4d) },
+   { "glMultiTexCoord4dv", _O(MultiTexCoord4dv) },
    { "glMultiTexCoord4fARB", _O(MultiTexCoord4fARB) },
    { "glMultiTexCoord4fvARB", _O(MultiTexCoord4fvARB) },
-   { "glMultiTexCoord4iARB", _O(MultiTexCoord4iARB) },
-   { "glMultiTexCoord4ivARB", _O(MultiTexCoord4ivARB) },
-   { "glMultiTexCoord4sARB", _O(MultiTexCoord4sARB) },
-   { "glMultiTexCoord4svARB", _O(MultiTexCoord4svARB) },
+   { "glMultiTexCoord4i", _O(MultiTexCoord4i) },
+   { "glMultiTexCoord4iv", _O(MultiTexCoord4iv) },
+   { "glMultiTexCoord4s", _O(MultiTexCoord4s) },
+   { "glMultiTexCoord4sv", _O(MultiTexCoord4sv) },
    { "glAttachShader", _O(AttachShader) },
    { "glCreateProgram", _O(CreateProgram) },
    { "glCreateShader", _O(CreateShader) },
@@ -1001,29 +1001,29 @@ const struct name_offset known_dispatch[] = {
    { "glFramebufferTexture", _O(FramebufferTexture) },
    { "glGetBufferParameteri64v", _O(GetBufferParameteri64v) },
    { "glGetInteger64i_v", _O(GetInteger64i_v) },
-   { "glLoadTransposeMatrixdARB", _O(LoadTransposeMatrixdARB) },
-   { "glLoadTransposeMatrixfARB", _O(LoadTransposeMatrixfARB) },
-   { "glMultTransposeMatrixdARB", _O(MultTransposeMatrixdARB) },
-   { "glMultTransposeMatrixfARB", _O(MultTransposeMatrixfARB) },
-   { "glSampleCoverageARB", _O(SampleCoverageARB) },
-   { "glCompressedTexImage1DARB", _O(CompressedTexImage1DARB) },
-   { "glCompressedTexImage2DARB", _O(CompressedTexImage2DARB) },
-   { "glCompressedTexImage3DARB", _O(CompressedTexImage3DARB) },
-   { "glCompressedTexSubImage1DARB", _O(CompressedTexSubImage1DARB) },
-   { "glCompressedTexSubImage2DARB", _O(CompressedTexSubImage2DARB) },
-   { "glCompressedTexSubImage3DARB", _O(CompressedTexSubImage3DARB) },
-   { "glGetCompressedTexImageARB", _O(GetCompressedTexImageARB) },
-   { "glDisableVertexAttribArrayARB", _O(DisableVertexAttribArrayARB) },
-   { "glEnableVertexAttribArrayARB", _O(EnableVertexAttribArrayARB) },
+   { "glLoadTransposeMatrixd", _O(LoadTransposeMatrixd) },
+   { "glLoadTransposeMatrixf", _O(LoadTransposeMatrixf) },
+   { "glMultTransposeMatrixd", _O(MultTransposeMatrixd) },
+   { "glMultTransposeMatrixf", _O(MultTransposeMatrixf) },
+   { "glSampleCoverage", _O(SampleCoverage) },
+   { "glCompressedTexImage1D", _O(CompressedTexImage1D) },
+   { "glCompressedTexImage2D", _O(CompressedTexImage2D) },
+   { "glCompressedTexImage3D", _O(CompressedTexImage3D) },
+   { "glCompressedTexSubImage1D", _O(CompressedTexSubImage1D) },
+   { "glCompressedTexSubImage2D", _O(CompressedTexSubImage2D) },
+   { "glCompressedTexSubImage3D", _O(CompressedTexSubImage3D) },
+   { "glGetCompressedTexImage", _O(GetCompressedTexImage) },
+   { "glDisableVertexAttribArray", _O(DisableVertexAttribArray) },
+   { "glEnableVertexAttribArray", _O(EnableVertexAttribArray) },
    { "glGetProgramEnvParameterdvARB", _O(GetProgramEnvParameterdvARB) },
    { "glGetProgramEnvParameterfvARB", _O(GetProgramEnvParameterfvARB) },
    { "glGetProgramLocalParameterdvARB", _O(GetProgramLocalParameterdvARB) },
    { "glGetProgramLocalParameterfvARB", _O(GetProgramLocalParameterfvARB) },
    { "glGetProgramStringARB", _O(GetProgramStringARB) },
    { "glGetProgramivARB", _O(GetProgramivARB) },
-   { "glGetVertexAttribdvARB", _O(GetVertexAttribdvARB) },
-   { "glGetVertexAttribfvARB", _O(GetVertexAttribfvARB) },
-   { "glGetVertexAttribivARB", _O(GetVertexAttribivARB) },
+   { "glGetVertexAttribdv", _O(GetVertexAttribdv) },
+   { "glGetVertexAttribfv", _O(GetVertexAttribfv) },
+   { "glGetVertexAttribiv", _O(GetVertexAttribiv) },
    { "glProgramEnvParameter4dARB", _O(ProgramEnvParameter4dARB) },
    { "glProgramEnvParameter4dvARB", _O(ProgramEnvParameter4dvARB) },
    { "glProgramEnvParameter4fARB", _O(ProgramEnvParameter4fARB) },
@@ -1033,116 +1033,116 @@ const struct name_offset known_dispatch[] = {
    { "glProgramLocalParameter4fARB", _O(ProgramLocalParameter4fARB) },
    { "glProgramLocalParameter4fvARB", _O(ProgramLocalParameter4fvARB) },
    { "glProgramStringARB", _O(ProgramStringARB) },
-   { "glVertexAttrib1dARB", _O(VertexAttrib1dARB) },
-   { "glVertexAttrib1dvARB", _O(VertexAttrib1dvARB) },
+   { "glVertexAttrib1d", _O(VertexAttrib1d) },
+   { "glVertexAttrib1dv", _O(VertexAttrib1dv) },
    { "glVertexAttrib1fARB", _O(VertexAttrib1fARB) },
    { "glVertexAttrib1fvARB", _O(VertexAttrib1fvARB) },
-   { "glVertexAttrib1sARB", _O(VertexAttrib1sARB) },
-   { "glVertexAttrib1svARB", _O(VertexAttrib1svARB) },
-   { "glVertexAttrib2dARB", _O(VertexAttrib2dARB) },
-   { "glVertexAttrib2dvARB", _O(VertexAttrib2dvARB) },
+   { "glVertexAttrib1s", _O(VertexAttrib1s) },
+   { "glVertexAttrib1sv", _O(VertexAttrib1sv) },
+   { "glVertexAttrib2d", _O(VertexAttrib2d) },
+   { "glVertexAttrib2dv", _O(VertexAttrib2dv) },
    { "glVertexAttrib2fARB", _O(VertexAttrib2fARB) },
    { "glVertexAttrib2fvARB", _O(VertexAttrib2fvARB) },
-   { "glVertexAttrib2sARB", _O(VertexAttrib2sARB) },
-   { "glVertexAttrib2svARB", _O(VertexAttrib2svARB) },
-   { "glVertexAttrib3dARB", _O(VertexAttrib3dARB) },
-   { "glVertexAttrib3dvARB", _O(VertexAttrib3dvARB) },
+   { "glVertexAttrib2s", _O(VertexAttrib2s) },
+   { "glVertexAttrib2sv", _O(VertexAttrib2sv) },
+   { "glVertexAttrib3d", _O(VertexAttrib3d) },
+   { "glVertexAttrib3dv", _O(VertexAttrib3dv) },
    { "glVertexAttrib3fARB", _O(VertexAttrib3fARB) },
    { "glVertexAttrib3fvARB", _O(VertexAttrib3fvARB) },
-   { "glVertexAttrib3sARB", _O(VertexAttrib3sARB) },
-   { "glVertexAttrib3svARB", _O(VertexAttrib3svARB) },
-   { "glVertexAttrib4NbvARB", _O(VertexAttrib4NbvARB) },
-   { "glVertexAttrib4NivARB", _O(VertexAttrib4NivARB) },
-   { "glVertexAttrib4NsvARB", _O(VertexAttrib4NsvARB) },
-   { "glVertexAttrib4NubARB", _O(VertexAttrib4NubARB) },
-   { "glVertexAttrib4NubvARB", _O(VertexAttrib4NubvARB) },
-   { "glVertexAttrib4NuivARB", _O(VertexAttrib4NuivARB) },
-   { "glVertexAttrib4NusvARB", _O(VertexAttrib4NusvARB) },
-   { "glVertexAttrib4bvARB", _O(VertexAttrib4bvARB) },
-   { "glVertexAttrib4dARB", _O(VertexAttrib4dARB) },
-   { "glVertexAttrib4dvARB", _O(VertexAttrib4dvARB) },
+   { "glVertexAttrib3s", _O(VertexAttrib3s) },
+   { "glVertexAttrib3sv", _O(VertexAttrib3sv) },
+   { "glVertexAttrib4Nbv", _O(VertexAttrib4Nbv) },
+   { "glVertexAttrib4Niv", _O(VertexAttrib4Niv) },
+   { "glVertexAttrib4Nsv", _O(VertexAttrib4Nsv) },
+   { "glVertexAttrib4Nub", _O(VertexAttrib4Nub) },
+   { "glVertexAttrib4Nubv", _O(VertexAttrib4Nubv) },
+   { "glVertexAttrib4Nuiv", _O(VertexAttrib4Nuiv) },
+   { "glVertexAttrib4Nusv", _O(VertexAttrib4Nusv) },
+   { "glVertexAttrib4bv", _O(VertexAttrib4bv) },
+   { "glVertexAttrib4d", _O(VertexAttrib4d) },
+   { "glVertexAttrib4dv", _O(VertexAttrib4dv) },
    { "glVertexAttrib4fARB", _O(VertexAttrib4fARB) },
    { "glVertexAttrib4fvARB", _O(VertexAttrib4fvARB) },
-   { "glVertexAttrib4ivARB", _O(VertexAttrib4ivARB) },
-   { "glVertexAttrib4sARB", _O(VertexAttrib4sARB) },
-   { "glVertexAttrib4svARB", _O(VertexAttrib4svARB) },
-   { "glVertexAttrib4ubvARB", _O(VertexAttrib4ubvARB) },
-   { "glVertexAttrib4uivARB", _O(VertexAttrib4uivARB) },
-   { "glVertexAttrib4usvARB", _O(VertexAttrib4usvARB) },
-   { "glVertexAttribPointerARB", _O(VertexAttribPointerARB) },
-   { "glBindBufferARB", _O(BindBufferARB) },
-   { "glBufferDataARB", _O(BufferDataARB) },
-   { "glBufferSubDataARB", _O(BufferSubDataARB) },
-   { "glDeleteBuffersARB", _O(DeleteBuffersARB) },
-   { "glGenBuffersARB", _O(GenBuffersARB) },
-   { "glGetBufferParameterivARB", _O(GetBufferParameterivARB) },
-   { "glGetBufferPointervARB", _O(GetBufferPointervARB) },
-   { "glGetBufferSubDataARB", _O(GetBufferSubDataARB) },
-   { "glIsBufferARB", _O(IsBufferARB) },
-   { "glMapBufferARB", _O(MapBufferARB) },
-   { "glUnmapBufferARB", _O(UnmapBufferARB) },
-   { "glBeginQueryARB", _O(BeginQueryARB) },
-   { "glDeleteQueriesARB", _O(DeleteQueriesARB) },
-   { "glEndQueryARB", _O(EndQueryARB) },
-   { "glGenQueriesARB", _O(GenQueriesARB) },
-   { "glGetQueryObjectivARB", _O(GetQueryObjectivARB) },
-   { "glGetQueryObjectuivARB", _O(GetQueryObjectuivARB) },
-   { "glGetQueryivARB", _O(GetQueryivARB) },
-   { "glIsQueryARB", _O(IsQueryARB) },
+   { "glVertexAttrib4iv", _O(VertexAttrib4iv) },
+   { "glVertexAttrib4s", _O(VertexAttrib4s) },
+   { "glVertexAttrib4sv", _O(VertexAttrib4sv) },
+   { "glVertexAttrib4ubv", _O(VertexAttrib4ubv) },
+   { "glVertexAttrib4uiv", _O(VertexAttrib4uiv) },
+   { "glVertexAttrib4usv", _O(VertexAttrib4usv) },
+   { "glVertexAttribPointer", _O(VertexAttribPointer) },
+   { "glBindBuffer", _O(BindBuffer) },
+   { "glBufferData", _O(BufferData) },
+   { "glBufferSubData", _O(BufferSubData) },
+   { "glDeleteBuffers", _O(DeleteBuffers) },
+   { "glGenBuffers", _O(GenBuffers) },
+   { "glGetBufferParameteriv", _O(GetBufferParameteriv) },
+   { "glGetBufferPointerv", _O(GetBufferPointerv) },
+   { "glGetBufferSubData", _O(GetBufferSubData) },
+   { "glIsBuffer", _O(IsBuffer) },
+   { "glMapBuffer", _O(MapBuffer) },
+   { "glUnmapBuffer", _O(UnmapBuffer) },
+   { "glBeginQuery", _O(BeginQuery) },
+   { "glDeleteQueries", _O(DeleteQueries) },
+   { "glEndQuery", _O(EndQuery) },
+   { "glGenQueries", _O(GenQueries) },
+   { "glGetQueryObjectiv", _O(GetQueryObjectiv) },
+   { "glGetQueryObjectuiv", _O(GetQueryObjectuiv) },
+   { "glGetQueryiv", _O(GetQueryiv) },
+   { "glIsQuery", _O(IsQuery) },
    { "glAttachObjectARB", _O(AttachObjectARB) },
-   { "glCompileShaderARB", _O(CompileShaderARB) },
+   { "glCompileShader", _O(CompileShader) },
    { "glCreateProgramObjectARB", _O(CreateProgramObjectARB) },
    { "glCreateShaderObjectARB", _O(CreateShaderObjectARB) },
    { "glDeleteObjectARB", _O(DeleteObjectARB) },
    { "glDetachObjectARB", _O(DetachObjectARB) },
-   { "glGetActiveUniformARB", _O(GetActiveUniformARB) },
+   { "glGetActiveUniform", _O(GetActiveUniform) },
    { "glGetAttachedObjectsARB", _O(GetAttachedObjectsARB) },
    { "glGetHandleARB", _O(GetHandleARB) },
    { "glGetInfoLogARB", _O(GetInfoLogARB) },
    { "glGetObjectParameterfvARB", _O(GetObjectParameterfvARB) },
    { "glGetObjectParameterivARB", _O(GetObjectParameterivARB) },
-   { "glGetShaderSourceARB", _O(GetShaderSourceARB) },
-   { "glGetUniformLocationARB", _O(GetUniformLocationARB) },
-   { "glGetUniformfvARB", _O(GetUniformfvARB) },
-   { "glGetUniformivARB", _O(GetUniformivARB) },
-   { "glLinkProgramARB", _O(LinkProgramARB) },
-   { "glShaderSourceARB", _O(ShaderSourceARB) },
-   { "glUniform1fARB", _O(Uniform1fARB) },
-   { "glUniform1fvARB", _O(Uniform1fvARB) },
-   { "glUniform1iARB", _O(Uniform1iARB) },
-   { "glUniform1ivARB", _O(Uniform1ivARB) },
-   { "glUniform2fARB", _O(Uniform2fARB) },
-   { "glUniform2fvARB", _O(Uniform2fvARB) },
-   { "glUniform2iARB", _O(Uniform2iARB) },
-   { "glUniform2ivARB", _O(Uniform2ivARB) },
-   { "glUniform3fARB", _O(Uniform3fARB) },
-   { "glUniform3fvARB", _O(Uniform3fvARB) },
-   { "glUniform3iARB", _O(Uniform3iARB) },
-   { "glUniform3ivARB", _O(Uniform3ivARB) },
-   { "glUniform4fARB", _O(Uniform4fARB) },
-   { "glUniform4fvARB", _O(Uniform4fvARB) },
-   { "glUniform4iARB", _O(Uniform4iARB) },
-   { "glUniform4ivARB", _O(Uniform4ivARB) },
-   { "glUniformMatrix2fvARB", _O(UniformMatrix2fvARB) },
-   { "glUniformMatrix3fvARB", _O(UniformMatrix3fvARB) },
-   { "glUniformMatrix4fvARB", _O(UniformMatrix4fvARB) },
-   { "glUseProgramObjectARB", _O(UseProgramObjectARB) },
-   { "glValidateProgramARB", _O(ValidateProgramARB) },
-   { "glBindAttribLocationARB", _O(BindAttribLocationARB) },
-   { "glGetActiveAttribARB", _O(GetActiveAttribARB) },
-   { "glGetAttribLocationARB", _O(GetAttribLocationARB) },
-   { "glDrawBuffersARB", _O(DrawBuffersARB) },
-   { "glClampColorARB", _O(ClampColorARB) },
+   { "glGetShaderSource", _O(GetShaderSource) },
+   { "glGetUniformLocation", _O(GetUniformLocation) },
+   { "glGetUniformfv", _O(GetUniformfv) },
+   { "glGetUniformiv", _O(GetUniformiv) },
+   { "glLinkProgram", _O(LinkProgram) },
+   { "glShaderSource", _O(ShaderSource) },
+   { "glUniform1f", _O(Uniform1f) },
+   { "glUniform1fv", _O(Uniform1fv) },
+   { "glUniform1i", _O(Uniform1i) },
+   { "glUniform1iv", _O(Uniform1iv) },
+   { "glUniform2f", _O(Uniform2f) },
+   { "glUniform2fv", _O(Uniform2fv) },
+   { "glUniform2i", _O(Uniform2i) },
+   { "glUniform2iv", _O(Uniform2iv) },
+   { "glUniform3f", _O(Uniform3f) },
+   { "glUniform3fv", _O(Uniform3fv) },
+   { "glUniform3i", _O(Uniform3i) },
+   { "glUniform3iv", _O(Uniform3iv) },
+   { "glUniform4f", _O(Uniform4f) },
+   { "glUniform4fv", _O(Uniform4fv) },
+   { "glUniform4i", _O(Uniform4i) },
+   { "glUniform4iv", _O(Uniform4iv) },
+   { "glUniformMatrix2fv", _O(UniformMatrix2fv) },
+   { "glUniformMatrix3fv", _O(UniformMatrix3fv) },
+   { "glUniformMatrix4fv", _O(UniformMatrix4fv) },
+   { "glUseProgram", _O(UseProgram) },
+   { "glValidateProgram", _O(ValidateProgram) },
+   { "glBindAttribLocation", _O(BindAttribLocation) },
+   { "glGetActiveAttrib", _O(GetActiveAttrib) },
+   { "glGetAttribLocation", _O(GetAttribLocation) },
+   { "glDrawBuffers", _O(DrawBuffers) },
+   { "glClampColor", _O(ClampColor) },
    { "glDrawArraysInstancedARB", _O(DrawArraysInstancedARB) },
    { "glDrawElementsInstancedARB", _O(DrawElementsInstancedARB) },
    { "glRenderbufferStorageMultisample", _O(RenderbufferStorageMultisample) },
-   { "glFramebufferTextureARB", _O(FramebufferTextureARB) },
+   { "glFramebufferTexture", _O(FramebufferTexture) },
    { "glFramebufferTextureFaceARB", _O(FramebufferTextureFaceARB) },
-   { "glProgramParameteriARB", _O(ProgramParameteriARB) },
-   { "glVertexAttribDivisorARB", _O(VertexAttribDivisorARB) },
+   { "glProgramParameteri", _O(ProgramParameteri) },
+   { "glVertexAttribDivisor", _O(VertexAttribDivisor) },
    { "glFlushMappedBufferRange", _O(FlushMappedBufferRange) },
    { "glMapBufferRange", _O(MapBufferRange) },
-   { "glTexBufferARB", _O(TexBufferARB) },
+   { "glTexBuffer", _O(TexBuffer) },
    { "glBindVertexArray", _O(BindVertexArray) },
    { "glGenVertexArrays", _O(GenVertexArrays) },
    { "glCopyBufferSubData", _O(CopyBufferSubData) },
@@ -1266,52 +1266,52 @@ const struct name_offset known_dispatch[] = {
    { "glNormalPointerEXT", _O(NormalPointerEXT) },
    { "glTexCoordPointerEXT", _O(TexCoordPointerEXT) },
    { "glVertexPointerEXT", _O(VertexPointerEXT) },
-   { "glPointParameterfEXT", _O(PointParameterfEXT) },
-   { "glPointParameterfvEXT", _O(PointParameterfvEXT) },
+   { "glPointParameterf", _O(PointParameterf) },
+   { "glPointParameterfv", _O(PointParameterfv) },
    { "glLockArraysEXT", _O(LockArraysEXT) },
    { "glUnlockArraysEXT", _O(UnlockArraysEXT) },
-   { "glSecondaryColor3bEXT", _O(SecondaryColor3bEXT) },
-   { "glSecondaryColor3bvEXT", _O(SecondaryColor3bvEXT) },
-   { "glSecondaryColor3dEXT", _O(SecondaryColor3dEXT) },
-   { "glSecondaryColor3dvEXT", _O(SecondaryColor3dvEXT) },
+   { "glSecondaryColor3b", _O(SecondaryColor3b) },
+   { "glSecondaryColor3bv", _O(SecondaryColor3bv) },
+   { "glSecondaryColor3d", _O(SecondaryColor3d) },
+   { "glSecondaryColor3dv", _O(SecondaryColor3dv) },
    { "glSecondaryColor3fEXT", _O(SecondaryColor3fEXT) },
    { "glSecondaryColor3fvEXT", _O(SecondaryColor3fvEXT) },
-   { "glSecondaryColor3iEXT", _O(SecondaryColor3iEXT) },
-   { "glSecondaryColor3ivEXT", _O(SecondaryColor3ivEXT) },
-   { "glSecondaryColor3sEXT", _O(SecondaryColor3sEXT) },
-   { "glSecondaryColor3svEXT", _O(SecondaryColor3svEXT) },
-   { "glSecondaryColor3ubEXT", _O(SecondaryColor3ubEXT) },
-   { "glSecondaryColor3ubvEXT", _O(SecondaryColor3ubvEXT) },
-   { "glSecondaryColor3uiEXT", _O(SecondaryColor3uiEXT) },
-   { "glSecondaryColor3uivEXT", _O(SecondaryColor3uivEXT) },
-   { "glSecondaryColor3usEXT", _O(SecondaryColor3usEXT) },
-   { "glSecondaryColor3usvEXT", _O(SecondaryColor3usvEXT) },
-   { "glSecondaryColorPointerEXT", _O(SecondaryColorPointerEXT) },
-   { "glMultiDrawArraysEXT", _O(MultiDrawArraysEXT) },
+   { "glSecondaryColor3i", _O(SecondaryColor3i) },
+   { "glSecondaryColor3iv", _O(SecondaryColor3iv) },
+   { "glSecondaryColor3s", _O(SecondaryColor3s) },
+   { "glSecondaryColor3sv", _O(SecondaryColor3sv) },
+   { "glSecondaryColor3ub", _O(SecondaryColor3ub) },
+   { "glSecondaryColor3ubv", _O(SecondaryColor3ubv) },
+   { "glSecondaryColor3ui", _O(SecondaryColor3ui) },
+   { "glSecondaryColor3uiv", _O(SecondaryColor3uiv) },
+   { "glSecondaryColor3us", _O(SecondaryColor3us) },
+   { "glSecondaryColor3usv", _O(SecondaryColor3usv) },
+   { "glSecondaryColorPointer", _O(SecondaryColorPointer) },
+   { "glMultiDrawArrays", _O(MultiDrawArrays) },
    { "glMultiDrawElementsEXT", _O(MultiDrawElementsEXT) },
-   { "glFogCoordPointerEXT", _O(FogCoordPointerEXT) },
-   { "glFogCoorddEXT", _O(FogCoorddEXT) },
-   { "glFogCoorddvEXT", _O(FogCoorddvEXT) },
+   { "glFogCoordPointer", _O(FogCoordPointer) },
+   { "glFogCoordd", _O(FogCoordd) },
+   { "glFogCoorddv", _O(FogCoorddv) },
    { "glFogCoordfEXT", _O(FogCoordfEXT) },
    { "glFogCoordfvEXT", _O(FogCoordfvEXT) },
-   { "glBlendFuncSeparateEXT", _O(BlendFuncSeparateEXT) },
+   { "glBlendFuncSeparate", _O(BlendFuncSeparate) },
    { "glResizeBuffersMESA", _O(ResizeBuffersMESA) },
-   { "glWindowPos2dMESA", _O(WindowPos2dMESA) },
-   { "glWindowPos2dvMESA", _O(WindowPos2dvMESA) },
-   { "glWindowPos2fMESA", _O(WindowPos2fMESA) },
-   { "glWindowPos2fvMESA", _O(WindowPos2fvMESA) },
-   { "glWindowPos2iMESA", _O(WindowPos2iMESA) },
-   { "glWindowPos2ivMESA", _O(WindowPos2ivMESA) },
-   { "glWindowPos2sMESA", _O(WindowPos2sMESA) },
-   { "glWindowPos2svMESA", _O(WindowPos2svMESA) },
-   { "glWindowPos3dMESA", _O(WindowPos3dMESA) },
-   { "glWindowPos3dvMESA", _O(WindowPos3dvMESA) },
-   { "glWindowPos3fMESA", _O(WindowPos3fMESA) },
-   { "glWindowPos3fvMESA", _O(WindowPos3fvMESA) },
-   { "glWindowPos3iMESA", _O(WindowPos3iMESA) },
-   { "glWindowPos3ivMESA", _O(WindowPos3ivMESA) },
-   { "glWindowPos3sMESA", _O(WindowPos3sMESA) },
-   { "glWindowPos3svMESA", _O(WindowPos3svMESA) },
+   { "glWindowPos2d", _O(WindowPos2d) },
+   { "glWindowPos2dv", _O(WindowPos2dv) },
+   { "glWindowPos2f", _O(WindowPos2f) },
+   { "glWindowPos2fv", _O(WindowPos2fv) },
+   { "glWindowPos2i", _O(WindowPos2i) },
+   { "glWindowPos2iv", _O(WindowPos2iv) },
+   { "glWindowPos2s", _O(WindowPos2s) },
+   { "glWindowPos2sv", _O(WindowPos2sv) },
+   { "glWindowPos3d", _O(WindowPos3d) },
+   { "glWindowPos3dv", _O(WindowPos3dv) },
+   { "glWindowPos3f", _O(WindowPos3f) },
+   { "glWindowPos3fv", _O(WindowPos3fv) },
+   { "glWindowPos3i", _O(WindowPos3i) },
+   { "glWindowPos3iv", _O(WindowPos3iv) },
+   { "glWindowPos3s", _O(WindowPos3s) },
+   { "glWindowPos3sv", _O(WindowPos3sv) },
    { "glWindowPos4dMESA", _O(WindowPos4dMESA) },
    { "glWindowPos4dvMESA", _O(WindowPos4dvMESA) },
    { "glWindowPos4fMESA", _O(WindowPos4fMESA) },
@@ -1323,20 +1323,20 @@ const struct name_offset known_dispatch[] = {
    { "glMultiModeDrawArraysIBM", _O(MultiModeDrawArraysIBM) },
    { "glMultiModeDrawElementsIBM", _O(MultiModeDrawElementsIBM) },
    { "glAreProgramsResidentNV", _O(AreProgramsResidentNV) },
-   { "glBindProgramNV", _O(BindProgramNV) },
-   { "glDeleteProgramsNV", _O(DeleteProgramsNV) },
+   { "glBindProgramARB", _O(BindProgramARB) },
+   { "glDeleteProgramsARB", _O(DeleteProgramsARB) },
    { "glExecuteProgramNV", _O(ExecuteProgramNV) },
-   { "glGenProgramsNV", _O(GenProgramsNV) },
+   { "glGenProgramsARB", _O(GenProgramsARB) },
    { "glGetProgramParameterdvNV", _O(GetProgramParameterdvNV) },
    { "glGetProgramParameterfvNV", _O(GetProgramParameterfvNV) },
    { "glGetProgramStringNV", _O(GetProgramStringNV) },
    { "glGetProgramivNV", _O(GetProgramivNV) },
    { "glGetTrackMatrixivNV", _O(GetTrackMatrixivNV) },
-   { "glGetVertexAttribPointervNV", _O(GetVertexAttribPointervNV) },
+   { "glGetVertexAttribPointerv", _O(GetVertexAttribPointerv) },
    { "glGetVertexAttribdvNV", _O(GetVertexAttribdvNV) },
    { "glGetVertexAttribfvNV", _O(GetVertexAttribfvNV) },
    { "glGetVertexAttribivNV", _O(GetVertexAttribivNV) },
-   { "glIsProgramNV", _O(IsProgramNV) },
+   { "glIsProgram", _O(IsProgram) },
    { "glLoadProgramNV", _O(LoadProgramNV) },
    { "glProgramParameters4dvNV", _O(ProgramParameters4dvNV) },
    { "glProgramParameters4fvNV", _O(ProgramParameters4fvNV) },
@@ -1400,60 +1400,60 @@ const struct name_offset known_dispatch[] = {
    { "glPassTexCoordATI", _O(PassTexCoordATI) },
    { "glSampleMapATI", _O(SampleMapATI) },
    { "glSetFragmentShaderConstantATI", _O(SetFragmentShaderConstantATI) },
-   { "glPointParameteriNV", _O(PointParameteriNV) },
-   { "glPointParameterivNV", _O(PointParameterivNV) },
+   { "glPointParameteri", _O(PointParameteri) },
+   { "glPointParameteriv", _O(PointParameteriv) },
    { "glActiveStencilFaceEXT", _O(ActiveStencilFaceEXT) },
    { "glBindVertexArrayAPPLE", _O(BindVertexArrayAPPLE) },
-   { "glDeleteVertexArraysAPPLE", _O(DeleteVertexArraysAPPLE) },
+   { "glDeleteVertexArrays", _O(DeleteVertexArrays) },
    { "glGenVertexArraysAPPLE", _O(GenVertexArraysAPPLE) },
-   { "glIsVertexArrayAPPLE", _O(IsVertexArrayAPPLE) },
+   { "glIsVertexArray", _O(IsVertexArray) },
    { "glGetProgramNamedParameterdvNV", _O(GetProgramNamedParameterdvNV) },
    { "glGetProgramNamedParameterfvNV", _O(GetProgramNamedParameterfvNV) },
    { "glProgramNamedParameter4dNV", _O(ProgramNamedParameter4dNV) },
    { "glProgramNamedParameter4dvNV", _O(ProgramNamedParameter4dvNV) },
    { "glProgramNamedParameter4fNV", _O(ProgramNamedParameter4fNV) },
    { "glProgramNamedParameter4fvNV", _O(ProgramNamedParameter4fvNV) },
-   { "glPrimitiveRestartIndexNV", _O(PrimitiveRestartIndexNV) },
+   { "glPrimitiveRestartIndex", _O(PrimitiveRestartIndex) },
    { "glPrimitiveRestartNV", _O(PrimitiveRestartNV) },
    { "glDepthBoundsEXT", _O(DepthBoundsEXT) },
-   { "glBlendEquationSeparateEXT", _O(BlendEquationSeparateEXT) },
-   { "glBindFramebufferEXT", _O(BindFramebufferEXT) },
-   { "glBindRenderbufferEXT", _O(BindRenderbufferEXT) },
-   { "glCheckFramebufferStatusEXT", _O(CheckFramebufferStatusEXT) },
-   { "glDeleteFramebuffersEXT", _O(DeleteFramebuffersEXT) },
-   { "glDeleteRenderbuffersEXT", _O(DeleteRenderbuffersEXT) },
-   { "glFramebufferRenderbufferEXT", _O(FramebufferRenderbufferEXT) },
-   { "glFramebufferTexture1DEXT", _O(FramebufferTexture1DEXT) },
-   { "glFramebufferTexture2DEXT", _O(FramebufferTexture2DEXT) },
-   { "glFramebufferTexture3DEXT", _O(FramebufferTexture3DEXT) },
-   { "glGenFramebuffersEXT", _O(GenFramebuffersEXT) },
-   { "glGenRenderbuffersEXT", _O(GenRenderbuffersEXT) },
-   { "glGenerateMipmapEXT", _O(GenerateMipmapEXT) },
-   { "glGetFramebufferAttachmentParameterivEXT", _O(GetFramebufferAttachmentParameterivEXT) },
-   { "glGetRenderbufferParameterivEXT", _O(GetRenderbufferParameterivEXT) },
-   { "glIsFramebufferEXT", _O(IsFramebufferEXT) },
-   { "glIsRenderbufferEXT", _O(IsRenderbufferEXT) },
-   { "glRenderbufferStorageEXT", _O(RenderbufferStorageEXT) },
-   { "glBlitFramebufferEXT", _O(BlitFramebufferEXT) },
+   { "glBlendEquationSeparate", _O(BlendEquationSeparate) },
+   { "glBindFramebuffer", _O(BindFramebuffer) },
+   { "glBindRenderbuffer", _O(BindRenderbuffer) },
+   { "glCheckFramebufferStatus", _O(CheckFramebufferStatus) },
+   { "glDeleteFramebuffers", _O(DeleteFramebuffers) },
+   { "glDeleteRenderbuffers", _O(DeleteRenderbuffers) },
+   { "glFramebufferRenderbuffer", _O(FramebufferRenderbuffer) },
+   { "glFramebufferTexture1D", _O(FramebufferTexture1D) },
+   { "glFramebufferTexture2D", _O(FramebufferTexture2D) },
+   { "glFramebufferTexture3D", _O(FramebufferTexture3D) },
+   { "glGenFramebuffers", _O(GenFramebuffers) },
+   { "glGenRenderbuffers", _O(GenRenderbuffers) },
+   { "glGenerateMipmap", _O(GenerateMipmap) },
+   { "glGetFramebufferAttachmentParameteriv", _O(GetFramebufferAttachmentParameteriv) },
+   { "glGetRenderbufferParameteriv", _O(GetRenderbufferParameteriv) },
+   { "glIsFramebuffer", _O(IsFramebuffer) },
+   { "glIsRenderbuffer", _O(IsRenderbuffer) },
+   { "glRenderbufferStorage", _O(RenderbufferStorage) },
+   { "glBlitFramebuffer", _O(BlitFramebuffer) },
    { "glBufferParameteriAPPLE", _O(BufferParameteriAPPLE) },
    { "glFlushMappedBufferRangeAPPLE", _O(FlushMappedBufferRangeAPPLE) },
-   { "glBindFragDataLocationEXT", _O(BindFragDataLocationEXT) },
-   { "glGetFragDataLocationEXT", _O(GetFragDataLocationEXT) },
-   { "glGetUniformuivEXT", _O(GetUniformuivEXT) },
-   { "glGetVertexAttribIivEXT", _O(GetVertexAttribIivEXT) },
-   { "glGetVertexAttribIuivEXT", _O(GetVertexAttribIuivEXT) },
-   { "glUniform1uiEXT", _O(Uniform1uiEXT) },
-   { "glUniform1uivEXT", _O(Uniform1uivEXT) },
-   { "glUniform2uiEXT", _O(Uniform2uiEXT) },
-   { "glUniform2uivEXT", _O(Uniform2uivEXT) },
-   { "glUniform3uiEXT", _O(Uniform3uiEXT) },
-   { "glUniform3uivEXT", _O(Uniform3uivEXT) },
-   { "glUniform4uiEXT", _O(Uniform4uiEXT) },
-   { "glUniform4uivEXT", _O(Uniform4uivEXT) },
+   { "glBindFragDataLocation", _O(BindFragDataLocation) },
+   { "glGetFragDataLocation", _O(GetFragDataLocation) },
+   { "glGetUniformuiv", _O(GetUniformuiv) },
+   { "glGetVertexAttribIiv", _O(GetVertexAttribIiv) },
+   { "glGetVertexAttribIuiv", _O(GetVertexAttribIuiv) },
+   { "glUniform1ui", _O(Uniform1ui) },
+   { "glUniform1uiv", _O(Uniform1uiv) },
+   { "glUniform2ui", _O(Uniform2ui) },
+   { "glUniform2uiv", _O(Uniform2uiv) },
+   { "glUniform3ui", _O(Uniform3ui) },
+   { "glUniform3uiv", _O(Uniform3uiv) },
+   { "glUniform4ui", _O(Uniform4ui) },
+   { "glUniform4uiv", _O(Uniform4uiv) },
    { "glVertexAttribI1iEXT", _O(VertexAttribI1iEXT) },
-   { "glVertexAttribI1ivEXT", _O(VertexAttribI1ivEXT) },
+   { "glVertexAttribI1iv", _O(VertexAttribI1iv) },
    { "glVertexAttribI1uiEXT", _O(VertexAttribI1uiEXT) },
-   { "glVertexAttribI1uivEXT", _O(VertexAttribI1uivEXT) },
+   { "glVertexAttribI1uiv", _O(VertexAttribI1uiv) },
    { "glVertexAttribI2iEXT", _O(VertexAttribI2iEXT) },
    { "glVertexAttribI2ivEXT", _O(VertexAttribI2ivEXT) },
    { "glVertexAttribI2uiEXT", _O(VertexAttribI2uiEXT) },
@@ -1462,38 +1462,38 @@ const struct name_offset known_dispatch[] = {
    { "glVertexAttribI3ivEXT", _O(VertexAttribI3ivEXT) },
    { "glVertexAttribI3uiEXT", _O(VertexAttribI3uiEXT) },
    { "glVertexAttribI3uivEXT", _O(VertexAttribI3uivEXT) },
-   { "glVertexAttribI4bvEXT", _O(VertexAttribI4bvEXT) },
+   { "glVertexAttribI4bv", _O(VertexAttribI4bv) },
    { "glVertexAttribI4iEXT", _O(VertexAttribI4iEXT) },
    { "glVertexAttribI4ivEXT", _O(VertexAttribI4ivEXT) },
-   { "glVertexAttribI4svEXT", _O(VertexAttribI4svEXT) },
-   { "glVertexAttribI4ubvEXT", _O(VertexAttribI4ubvEXT) },
+   { "glVertexAttribI4sv", _O(VertexAttribI4sv) },
+   { "glVertexAttribI4ubv", _O(VertexAttribI4ubv) },
    { "glVertexAttribI4uiEXT", _O(VertexAttribI4uiEXT) },
    { "glVertexAttribI4uivEXT", _O(VertexAttribI4uivEXT) },
-   { "glVertexAttribI4usvEXT", _O(VertexAttribI4usvEXT) },
-   { "glVertexAttribIPointerEXT", _O(VertexAttribIPointerEXT) },
-   { "glFramebufferTextureLayerEXT", _O(FramebufferTextureLayerEXT) },
-   { "glColorMaskIndexedEXT", _O(ColorMaskIndexedEXT) },
-   { "glDisableIndexedEXT", _O(DisableIndexedEXT) },
-   { "glEnableIndexedEXT", _O(EnableIndexedEXT) },
-   { "glGetBooleanIndexedvEXT", _O(GetBooleanIndexedvEXT) },
-   { "glGetIntegerIndexedvEXT", _O(GetIntegerIndexedvEXT) },
-   { "glIsEnabledIndexedEXT", _O(IsEnabledIndexedEXT) },
+   { "glVertexAttribI4usv", _O(VertexAttribI4usv) },
+   { "glVertexAttribIPointer", _O(VertexAttribIPointer) },
+   { "glFramebufferTextureLayer", _O(FramebufferTextureLayer) },
+   { "glColorMaski", _O(ColorMaski) },
+   { "glDisablei", _O(Disablei) },
+   { "glEnablei", _O(Enablei) },
+   { "glGetBooleani_v", _O(GetBooleani_v) },
+   { "glGetIntegeri_v", _O(GetIntegeri_v) },
+   { "glIsEnabledi", _O(IsEnabledi) },
    { "glClearColorIiEXT", _O(ClearColorIiEXT) },
    { "glClearColorIuiEXT", _O(ClearColorIuiEXT) },
-   { "glGetTexParameterIivEXT", _O(GetTexParameterIivEXT) },
-   { "glGetTexParameterIuivEXT", _O(GetTexParameterIuivEXT) },
-   { "glTexParameterIivEXT", _O(TexParameterIivEXT) },
-   { "glTexParameterIuivEXT", _O(TexParameterIuivEXT) },
-   { "glBeginConditionalRenderNV", _O(BeginConditionalRenderNV) },
-   { "glEndConditionalRenderNV", _O(EndConditionalRenderNV) },
-   { "glBeginTransformFeedbackEXT", _O(BeginTransformFeedbackEXT) },
-   { "glBindBufferBaseEXT", _O(BindBufferBaseEXT) },
+   { "glGetTexParameterIiv", _O(GetTexParameterIiv) },
+   { "glGetTexParameterIuiv", _O(GetTexParameterIuiv) },
+   { "glTexParameterIiv", _O(TexParameterIiv) },
+   { "glTexParameterIuiv", _O(TexParameterIuiv) },
+   { "glBeginConditionalRender", _O(BeginConditionalRender) },
+   { "glEndConditionalRender", _O(EndConditionalRender) },
+   { "glBeginTransformFeedback", _O(BeginTransformFeedback) },
+   { "glBindBufferBase", _O(BindBufferBase) },
    { "glBindBufferOffsetEXT", _O(BindBufferOffsetEXT) },
-   { "glBindBufferRangeEXT", _O(BindBufferRangeEXT) },
-   { "glEndTransformFeedbackEXT", _O(EndTransformFeedbackEXT) },
-   { "glGetTransformFeedbackVaryingEXT", _O(GetTransformFeedbackVaryingEXT) },
-   { "glTransformFeedbackVaryingsEXT", _O(TransformFeedbackVaryingsEXT) },
-   { "glProvokingVertexEXT", _O(ProvokingVertexEXT) },
+   { "glBindBufferRange", _O(BindBufferRange) },
+   { "glEndTransformFeedback", _O(EndTransformFeedback) },
+   { "glGetTransformFeedbackVarying", _O(GetTransformFeedbackVarying) },
+   { "glTransformFeedbackVaryings", _O(TransformFeedbackVaryings) },
+   { "glProvokingVertex", _O(ProvokingVertex) },
    { "glGetObjectParameterivAPPLE", _O(GetObjectParameterivAPPLE) },
    { "glObjectPurgeableAPPLE", _O(ObjectPurgeableAPPLE) },
    { "glObjectUnpurgeableAPPLE", _O(ObjectUnpurgeableAPPLE) },
@@ -1504,14 +1504,14 @@ const struct name_offset known_dispatch[] = {
    { "glStencilFuncSeparateATI", _O(StencilFuncSeparateATI) },
    { "glProgramEnvParameters4fvEXT", _O(ProgramEnvParameters4fvEXT) },
    { "glProgramLocalParameters4fvEXT", _O(ProgramLocalParameters4fvEXT) },
-   { "glGetQueryObjecti64vEXT", _O(GetQueryObjecti64vEXT) },
-   { "glGetQueryObjectui64vEXT", _O(GetQueryObjectui64vEXT) },
+   { "glGetQueryObjecti64v", _O(GetQueryObjecti64v) },
+   { "glGetQueryObjectui64v", _O(GetQueryObjectui64v) },
    { "glEGLImageTargetRenderbufferStorageOES", _O(EGLImageTargetRenderbufferStorageOES) },
    { "glEGLImageTargetTexture2DOES", _O(EGLImageTargetTexture2DOES) },
    { NULL, 0 }
 };
 
-#ifndef GLX_INDIRECT_RENDERING
+#ifdef GLX_INDIRECT_RENDERING
 extern "C" {
 GLAPI GLboolean GLAPIENTRY
 glAreTexturesResidentEXT(GLsizei n, const GLuint *textures,
